/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.authentication.callbacks.jee;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import net.sf.jguard.authentication.http.HttpAuthenticationUtils;
import net.sf.jguard.authentication.http.HttpConstants;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.callbacks.CertificatesCallback;
import net.sf.jguard.ext.authentication.callbacks.JCaptchaCallback;
import net.sf.jguard.ext.authentication.certificates.CertificateConverter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bouncycastle.util.encoders.Base64;

import com.octo.captcha.service.CaptchaService;

/**
 * @author <a href="mailto:diabolo512@users.sourceforge.net ">Charles Gay</a>
 */
public class HttpCallbackHandler implements CallbackHandler{

	private static final String JAVAX_SERVLET_REQUEST_X509CERTIFICATE = "javax.servlet.request.X509Certificate";
	private static final String DIGEST_REALM = "Digest realm=\"";
	private static final String AUTHORIZATION = "Authorization";
	private static final String BASIC_REALM = "Basic realm=\"";
	private static final String NO_CACHE_AUTHORIZATION = "no-cache=\"Authorization\"";
	private static final String CACHE_CONTROL = "Cache-Control";
	private static final String WWW_AUTHENTICATE = "WWW-Authenticate";
	private static final String BASIC = "Basic ";
	private static final String ISO_8859_1 = "ISO-8859-1";
	/** Logger for this class */
	private static final Log logger = LogFactory.getLog(HttpCallbackHandler.class);
	private HttpServletRequest httpRequest;
	private HttpServletResponse httpResponse;
	private String authScheme=HttpConstants.FORM_AUTH;
	private static String loginField="login";
	private static String passwordField="password";
	private boolean afterRegistration;


	/**
	 * constructor required by javadoc of the CallbackHandler interface.
	 */
	public HttpCallbackHandler(){

	}

	/**
	 * constructor.
	 * @param request
	 * @param response
	 * @param authScheme
	 */
	public HttpCallbackHandler(HttpServletRequest request,HttpServletResponse response,String authScheme){
		this.httpRequest = request;
		this.httpResponse = response;
		this.authScheme = authScheme;


	}



	/**
	 * extract from the HttpServletRequest client's credentials.
	 * if those are not recognised, we put the challenge in the
	 * HttpServletResponse.
	 */
	public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
            boolean httpRelatedAuthScheme = false;
			logger.debug("authSchemeItem="+authScheme);
			String[] schemes = authScheme.split(",");
			List authSchemes = Arrays.asList(schemes);
			Iterator itAutSchemes = authSchemes.iterator();
			while(itAutSchemes.hasNext()){
				String scheme = (String)itAutSchemes.next();
				//FORM, BASIC, and DIGEST are mutual exclusive
				if(!httpRelatedAuthScheme && HttpConstants.FORM_AUTH.equalsIgnoreCase(scheme)){
					grabFormCredentials(this.httpRequest,callbacks);
					httpRelatedAuthScheme = true;
				}else if(!httpRelatedAuthScheme && HttpConstants.BASIC_AUTH.equalsIgnoreCase(scheme)){
					grabBasicCredentials(this.httpRequest,callbacks);
					httpRelatedAuthScheme = true;
				}else if(!httpRelatedAuthScheme && HttpConstants.DIGEST_AUTH.equalsIgnoreCase(scheme)){
					grabDigestCredentials(this.httpRequest,callbacks);
					httpRelatedAuthScheme = true;
				}
				//CLIENT_CERT can be used with another authentication mechanism
				//defined above
				if(HttpConstants.CLIENT_CERT_AUTH.equalsIgnoreCase(scheme)){
					grabClientCertCredentials(this.httpRequest,callbacks);
				}
			}

	}

	public HttpServletRequest getHttpRequest() {
		return httpRequest;
	}


	public void setHttpRequest(HttpServletRequest httpRequest) {
		this.httpRequest = httpRequest;
	}

	public HttpServletResponse getHttpResponse() {
		return httpResponse;
	}

	public void setHttpResponse(HttpServletResponse httpResponse) {
		this.httpResponse = httpResponse;
	}

	/**
	 * construct a header value to simulate a Basic authentication with the provided credentials.
	 * @param login
	 * @param password
	 * @param encoding
	 * @return header
	 */
	public static String buildBasicAuthHeader(String login,String password,String encoding){
		if(encoding==null){
			encoding=HttpCallbackHandler.ISO_8859_1;
		}
		StringBuffer decodedString = new StringBuffer();
		decodedString.append(login);
		decodedString.append(" : ");
		decodedString.append(password);
		String encodedString;
		try {
			encodedString = new String(Base64.encode(decodedString.toString().getBytes(encoding)));
		} catch (UnsupportedEncodingException e) {
			encodedString = new String(Base64.encode(decodedString.toString().getBytes()));
		}
		StringBuffer header = new StringBuffer();
		header.append(HttpCallbackHandler.BASIC);
		header.append(encodedString);
		header.append("==");
		return header.toString();
	}

	/**
	 * send to the client the BASIC challenge into the response, according to the RFC 2617.
	 * @param response reponse send to the Client
	 * @param realmName realm owned by the server => specify what kind of credential the user should provide
	 */
	public static void buildBasicChallenge(HttpServletResponse response,String realmName){
		StringBuffer responseValue= new StringBuffer();
		responseValue.append(HttpCallbackHandler.BASIC_REALM);
		responseValue.append(realmName);
		responseValue.append("\"");
		response.setHeader(HttpCallbackHandler.WWW_AUTHENTICATE,responseValue.toString());
		response.setHeader(HttpCallbackHandler.CACHE_CONTROL, HttpCallbackHandler.NO_CACHE_AUTHORIZATION);
		response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
	}

	/**
	 * parse into the HttpServletRequest the user and password field,
	 * and authenticate the user with these credentials using the <b>NON SECURE</> BASIC method.
	 * @param request request send by the client.
	 * @param callbacks
	 * @return authentication's result. <i>true</i> for authentication success, <i>false</i> otherwise.
	 */
	private boolean grabBasicCredentials(HttpServletRequest request,Callback[] callbacks){
		boolean result = false;
        String login="";
        String password="";
		//user and password are encoded in Base64
		String encodedLoginAndPwd = request.getHeader(HttpCallbackHandler.AUTHORIZATION);

		if(encodedLoginAndPwd==null ||encodedLoginAndPwd.equals("")){
			login =SecurityConstants.GUEST;
	        password =SecurityConstants.GUEST;

		}else{
			encodedLoginAndPwd = encodedLoginAndPwd.substring(6).trim();
			String decodedLoginAndPassword = null;

				String encoding =  request.getCharacterEncoding();
				if(encoding==null){
					encoding=HttpCallbackHandler.ISO_8859_1;
				}
				logger.debug(encoding);

				try {
					decodedLoginAndPassword = new String(Base64.decode(encodedLoginAndPwd.getBytes()),encoding);
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
					logger.error(" encoding "+encoding+" is not supported by the platform ");
				}

			String[] parts = decodedLoginAndPassword.split(":");
			if(parts.length == 2 ){
				login = parts[0].trim();
				password = parts[1].trim();

				result = true;
			}
			if((login=="" && password=="")||(parts.length==0)){
				login =SecurityConstants.GUEST;
		        password =SecurityConstants.GUEST;
			}

		}

		fillBasicCredentials(callbacks,login,password);
		return result;
	}

	 /**
	 * grab user credentials from request in the 'form' authentication metod.
     * @param request request send by the client
     * @param callbacks
     * @return authentication result : <b>true</b> when authentication succeed,<b>false</b> when authentication fails.
     */
	private  boolean  grabFormCredentials(HttpServletRequest request,Callback[] callbacks){
		boolean result = false;
        HttpSession session = request.getSession();

		for(int i=0;i<callbacks.length;i++){
        	if(callbacks[i] instanceof NameCallback){
        		NameCallback nc = (NameCallback)callbacks[i];
        		String login =httpRequest.getParameter(loginField);
        		nc.setName(login);
        	}else if(callbacks[i] instanceof PasswordCallback){
        		PasswordCallback pc = (PasswordCallback)callbacks[i];
        		String strPwd = httpRequest.getParameter(passwordField);
                 if(strPwd!= null &&strPwd!=""){
        		  pc.setPassword(strPwd.toCharArray());
                 }else{
                  pc.setPassword(null);
                 }
        	}else if(callbacks[i] instanceof JCaptchaCallback){
        		JCaptchaCallback pc = (JCaptchaCallback)callbacks[i];
        		pc.setCaptchaAnswer(httpRequest.getParameter(SecurityConstants.CAPTCHA_ANSWER));
        		pc.setCaptchaService((CaptchaService)session.getServletContext().getAttribute(SecurityConstants.CAPTCHA_SERVICE));
        		Subject subject = ((HttpAuthenticationUtils)session.getAttribute(HttpConstants.AUTH_UTILS)).getSubject();
        		if(subject==null ||this.isAfterRegistration()){
        		    pc.setSkipJCaptchaChallenge(true);
        		}

        		pc.setSessionID(session.getId());
        	}
        }
        result = true;


		return result;
	}

	 /**
	 * grab user credentials from request in the 'digest' authentication metod.
     * @param request request send by the client
     * @param callbacks
     * @return authentication result : <b>true</b> when authentication succeed,<b>false</b> when authentication fails.
     */
	private  boolean  grabDigestCredentials(HttpServletRequest request,Callback[] callbacks){
        boolean result = false;
		String login = "";
        String password = "";
        //all users must be authenticated
        //unless when the user send a wrong  login or/and password
        //=> he is redirected to the logonPage
        if(login==null || password == null ){
           login =SecurityConstants.GUEST;
           password =SecurityConstants.GUEST;
        }else{
            //TODO implements digest authentication
           result = true;
        }
		return result;
	}

	/**
	 * grab user credentials from request in the 'clientCert' authentication metod.
	 * @param request
	 * @param callbacks
	 * @return <code>true</code> if successfull, <code>false</code> otherwise
	 */
	private boolean grabClientCertCredentials(HttpServletRequest request,Callback[] callbacks) {
		if(!request.isSecure()){
			logger.warn(" certificate-based authentication MUST be do in secure mode ");
			logger.warn(" but connection is do with the non secured protocol "+request.getScheme());
			return false;
		}

		X509Certificate[] certificates = null;
		javax.security.cert.X509Certificate[] oldCerts = null;
		Object[] objects = (Object[]) request.getAttribute(HttpCallbackHandler.JAVAX_SERVLET_REQUEST_X509CERTIFICATE);
		if(objects instanceof X509Certificate[]) {
			certificates= (X509Certificate[]) objects;
		//convert old X509 certificates into new X509 certificates
		}else if(objects instanceof javax.security.cert.X509Certificate[]) {
			oldCerts = (javax.security.cert.X509Certificate[])objects;
			List newCerts = null;
			for(int i =0;i<oldCerts.length;i++){
			 newCerts = Arrays.asList(certificates);
			 newCerts.add(CertificateConverter.convertOldToNew(oldCerts[i]));
			}
			certificates = (X509Certificate[]) newCerts.toArray();
		}else{
			logger.warn(" X509certificates are needed but not provided by the client ");
			return false;
		}
		fillCredentials(callbacks,certificates);

		return true;
	}

	private void fillBasicCredentials(Callback[] callbacks, String login, String password) {
		for(int i=0;i<callbacks.length;i++){
        	if(callbacks[i] instanceof NameCallback){
        		NameCallback nc = (NameCallback)callbacks[i];
        		nc.setName(login);

        	}else if(callbacks[i] instanceof PasswordCallback){
        		PasswordCallback pc = (PasswordCallback)callbacks[i];
        		pc.setPassword(password.toCharArray());
        	}
        }
	}


	private void fillCredentials(Callback[] callbacks,X509Certificate[] certificates) {
		for(int i=0;i<callbacks.length;i++){
        	if(callbacks[i] instanceof CertificatesCallback){
        		CertificatesCallback cc = (CertificatesCallback)callbacks[i];
        		cc.setCertificates(certificates);
        		break;
        	}
        }
	}

	/**
	 * send to the client the DIGEST challenge into the response, according to the RFC 2617.
	 * @param response reponse send to the Client
	 * @param token realm owned by the server => specify what kind of credential the user should provide
	 */
	public void buildDigestChallenge(HttpServletResponse response, String token) {

			StringBuffer responseValue= new StringBuffer();
			responseValue.append(HttpCallbackHandler.DIGEST_REALM);
			responseValue.append(token);
			responseValue.append("\"");
			response.setHeader(HttpCallbackHandler.WWW_AUTHENTICATE,responseValue.toString());
			response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
	}


	/**
	 * gets the HttpRequest password field
	 * @return password field
	 */
	public static String getPasswordField() {
		return passwordField;
	}

	public static void setPasswordField(String passwordField) {
		if(passwordField!=null){
			HttpCallbackHandler.passwordField = passwordField;
		}
	}

	/**
	 * gets the HttpRequest login field
	 * @return login field
	 */
	public static String getLoginField() {
		return loginField;
	}

	public static void setLoginField(String loginField) {
		if(loginField!=null){
			HttpCallbackHandler.loginField = loginField;
		}
	}

	public void setAfterRegistration(boolean afterRegistration) {
       this.afterRegistration = afterRegistration;
	}

	public boolean isAfterRegistration() {
		return afterRegistration;
	}
}
