/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.

http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.listeners;

import javax.servlet.http.HttpSessionAttributeListener;
import javax.servlet.http.HttpSessionBindingEvent;

import net.sf.jguard.authentication.http.HttpAuthenticationUtils;
import net.sf.jguard.authentication.http.HttpConstants;
import net.sf.jguard.ext.audit.AuditManager;

/**
 * Audit JGuard Subject of http sessions, generating log for Subject changes
 *
 * @author Frederico Borelli
 *
 * @see net.sf.jguard.ext.audit.AuditManager
 */
public class SessionAttributeListener implements HttpSessionAttributeListener {

	public void attributeAdded(HttpSessionBindingEvent event) {
		if (event.getName().equals(HttpConstants.AUTH_UTILS)) {
			AuditManager.addEvent(((HttpAuthenticationUtils) event.getValue()).getSubject(), " was created ", " subject available to webapp ");
		}
	}

	public void attributeRemoved(HttpSessionBindingEvent event) {
		if (event.getName().equals(HttpConstants.AUTH_UTILS)) {
			AuditManager.addEvent(((HttpAuthenticationUtils) event.getValue()).getSubject(), " was removed ", " subject not available anymore to webapp ");
		}
	}

	public void attributeReplaced(HttpSessionBindingEvent event) {
		if (event.getName().equals(HttpConstants.AUTH_UTILS)) {
			AuditManager.addEvent(((HttpAuthenticationUtils) event.getValue()).getSubject(), " was modified ", " a different subject is now available to webapp ");
		}
	}

}
