/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.taglib;




import javax.security.auth.Subject;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.jstl.core.ConditionalTagSupport;

import net.sf.jguard.core.authorization.permissions.PermissionUtils;
import net.sf.jguard.core.authorization.permissions.URLPermission;
import net.sf.jguard.ext.authorization.AuthorizationUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.taglibs.standard.lang.support.ExpressionEvaluatorManager;


/**
 * display the jsp fragment if the user has got the right to access to the ressource
 * protected by the permission.
 * @author <a href="mailto:diabolo512@users.sourceforge.net ">Charles Gay</a>
 */
public class Authorized extends ConditionalTagSupport{
	/** Logger for this class */
	private static final Log logger = LogFactory.getLog(Authorized.class);

	/**
	* serial version id.
	*/
	private static final long serialVersionUID = 3833742183621736755L;
	private String uri;
	private String permission=URLPermission.class.getName();


    /**
     * @return uri
     */
    public String getUri() {
    	return uri;
    }


    /**
     * @param strUri
     */
    public void setUri(String strUri) {
              uri = strUri;

    }


	/**
     * allow or not to display jsp content;depends on access rights.
     * @return true if tag displays content when user is authorized; false otherwise
	 * @see javax.servlet.jsp.jstl.core.ConditionalTagSupport#condition()
	 */
	protected boolean condition() throws JspTagException {

        try {
			this.uri=(String)ExpressionEvaluatorManager.evaluate ("uri", this.uri, String.class, this, pageContext);
			String perm = (String)ExpressionEvaluatorManager.evaluate ("permission", this.permission, String.class, this, pageContext);
			if(perm!= null && !perm.equals("")){
				permission = perm;
			}
		} catch (JspException e1) {
			logger.error("condition()", e1);
            throw new JspTagException(e1.getMessage());
		}

		if(logger.isDebugEnabled()){
			logger.debug("<jguard:authorized> tag uri="+uri);
		}

		Subject subject = TagUtils.getSubject(this.pageContext);
		if(subject == null){
			return false;
		}

        StringBuffer actions = new StringBuffer();
        actions.append(uri);

        URLPermission urlPermission = (URLPermission)PermissionUtils.getPermission(permission,"dummy name",actions.toString());
        if(logger.isDebugEnabled()){
        	logger.debug("permission implementation class="+permission);
        	logger.debug("permission actions="+actions.toString());
        	logger.debug("URLPermission="+urlPermission);
        }
        if(!AuthorizationUtils.permitted(subject,urlPermission)){

              return false;

        }

              return true;

	}


	public String getPermission() {
		return permission;
	}


	public void setPermission(String permission) {
		this.permission = permission;
	}
}
